package de.hdm_stuttgart.mi.utils.xmlparse;

import gnu.getopt.LongOpt;

import java.util.Hashtable;
import java.util.Map;
import java.util.Vector;

/**
 * Practical extension to gnu's opt package
 *
*/
public class LongoptExplanation extends LongOpt {

  /**
   * @param isRequired true if parameter is being required, false otherwise.
   * @param name Parameter's name
   * @param flag Getopt flags
   * @param val Parameter's value
   * @throws IllegalArgumentException
   */
  public static void add
    (boolean isRequired,
     String name,
     StringBuffer flag,
     int val) throws IllegalArgumentException{
    addOption(name, new LongoptExplanation(isRequired, name, flag, val));
  }
  /**
   * @param isRequired true if parameter is being required, false otherwise.
   * @param name Parameter's name
   * @param argumentRequiredFlag T
   * @param flag Getopt flags
   * @param val Parameter's value
   * @param explain Explanation to CLI end user.
   * @throws IllegalArgumentException
   */
  public static void add
    (boolean isRequired,
     String name,
     int argumentRequiredFlag,
     StringBuffer flag,
     int val,
     final String explain) throws IllegalArgumentException{
    addOption(name, new LongoptExplanation(isRequired, name, argumentRequiredFlag, flag, val, explain));
  }
  private static void addOption(final String name, final LongoptExplanation option){
    if (null != optionOverName.get(name)){
      System.err.println("Option \"" + name + "\" already defined");
      System.exit(1);
    } else {
      optionsInOrder.add(option);
      optionOverName.put(name, option);
    }
  }
  /**
   * <p>Option values.</p>
   * @return Set of option values being defined.
   */
  public static LongoptExplanation [] getOptions(){
    return optionOverName.values().toArray(new LongoptExplanation[0]);
  }
  /**
   * <p>Short option list</p>
   * @return short options
   */
  public static String getShortOptString(){
    final StringBuffer shortOptString = new StringBuffer();
    final LongoptExplanation [] options = getOptions();

    for (int i = 0; i < options.length; i++){
      
      shortOptString.append((char)options[i].getVal());
      if (LongOpt.REQUIRED_ARGUMENT == options[i].getHasArg()){
        shortOptString.append(":");
      } 
      if (LongOpt.OPTIONAL_ARGUMENT == options[i].getHasArg()){
        shortOptString.append("::");
      }
    }
    return shortOptString.toString();
  }
  /**
   * <p>End user CLI usage description.</p>
   * @return Usage description
   */
  public static String getUsage(){
    StringBuffer ret = new StringBuffer();
    for (int i = 0; i < optionsInOrder.size(); i++){
      LongoptExplanation option = optionsInOrder.get(i);
      ret.append("\n  ");
      ret.append(option.getUsageComponent());
    }
    return ret.toString();
  }
  private String getUsageComponent(){
    StringBuffer res = new StringBuffer();
    if (!isRequiredArgument){
      res.append("[");
    }
      res.append("-");
      res.append((char)getVal());
    res.append(",--");
    res.append(getName());
    switch(getHasArg()){
    case NO_ARGUMENT:
      break;
    case REQUIRED_ARGUMENT:
      res.append(" ");
      res.append(argumentExplanation);
      break;
    case OPTIONAL_ARGUMENT:
      res.append(" [");
      res.append(argumentExplanation);
      res.append("]");
      break;
    default:
      System.err.println("No such argument code:" + getHasArg());
      System.exit(1);
    }
    if (!isRequiredArgument){
      res.append("]");
    }
    return res.toString();
  }
  private LongoptExplanation
    (boolean isRequired,
     String name,
     StringBuffer flag,
     int val) throws IllegalArgumentException{
    super(name, LongOpt.NO_ARGUMENT, flag, val);
    isRequiredArgument = isRequired;
  }
  private LongoptExplanation
    (boolean isRequired,
     String name,
     int argumentRequiredFlag,
     StringBuffer flag,
     int val,
     final String explain) throws IllegalArgumentException{
    super(name, argumentRequiredFlag, flag, val);
    isRequiredArgument = isRequired;
    argumentExplanation = explain;
  }
  private boolean isRequiredArgument;
  private String argumentExplanation;
  static private Map <String, LongoptExplanation> optionOverName = new Hashtable<String, LongoptExplanation> ();
  static private Vector<LongoptExplanation> optionsInOrder = new Vector<LongoptExplanation> ();
}
